@@ -45,7 +45,7 @@
  */
 
 #define FALLBACK_NAME _("Untitled window")
-#define ALL_WORKSPACES (0xFFFFFFFF)
+#define ALL_WORKSPACES (G_MAXINT)
 
 static GHashTable *window_hash = NULL;
 
@@ -163,6 +163,7 @@ struct _WnckWindowPrivate
   guint need_emit_icon_changed : 1;
   guint need_emit_class_changed : 1;
   guint need_emit_role_changed : 1;
+  guint need_emit_type_changed : 1;
 };
 
 G_DEFINE_TYPE (WnckWindow, wnck_window, G_TYPE_OBJECT);
@@ -177,11 +178,10 @@ enum {
   GEOMETRY_CHANGED,
   CLASS_CHANGED,
   ROLE_CHANGED,
+  TYPE_CHANGED,
   LAST_SIGNAL
 };
 
-static void wnck_window_init        (WnckWindow      *window);
-static void wnck_window_class_init  (WnckWindowClass *klass);
 static void wnck_window_finalize    (GObject        *object);
 
 static void emit_name_changed      (WnckWindow      *window);
@@ -196,6 +196,7 @@ static void emit_actions_changed   (WnckWindow       *window,
 static void emit_geometry_changed  (WnckWindow      *window);
 static void emit_class_changed     (WnckWindow      *window);
 static void emit_role_changed      (WnckWindow      *window);
+static void emit_type_changed      (WnckWindow      *window);
 
 static void update_name      (WnckWindow *window);
 static void update_state     (WnckWindow *window);
@@ -375,6 +376,22 @@ wnck_window_class_init (WnckWindowClass *klass)
                   G_STRUCT_OFFSET (WnckWindowClass, role_changed),
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
+
+  /**
+   * WnckWindow::type-changed:
+   * @window: the #WnckWindow which emitted the signal.
+   *
+   * Emitted when the EWMH type hint of the window changes.
+   *
+   * Since: 3.20
+   */
+  signals[TYPE_CHANGED] =
+    g_signal_new ("type_changed",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (WnckWindowClass, type_changed),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
 }
 
 static void
@@ -543,6 +560,7 @@ _wnck_window_create (Window      xwindow,
   window->priv->need_emit_icon_changed = FALSE;
   window->priv->need_emit_class_changed = FALSE;
   window->priv->need_emit_role_changed = FALSE;
+  window->priv->need_emit_type_changed = FALSE;
   force_update_now (window);
 
   return window;
@@ -1001,6 +1019,8 @@ wnck_window_set_window_type (WnckWindow *window, WnckWindowType wintype)
 		   (guchar *)&atom, 1);
 
   _wnck_error_trap_pop (display);
+
+  emit_type_changed (window);
 }
 
 /**
@@ -2763,6 +2783,7 @@ update_state (WnckWindow *window)
       break;
 
     case WNCK_WINDOW_NORMAL:
+    default:
       break;
     }
 
@@ -2782,6 +2803,7 @@ update_state (WnckWindow *window)
     case WNCK_WINDOW_NORMAL:
     case WNCK_WINDOW_DIALOG:
     case WNCK_WINDOW_UTILITY:
+    default:
       break;
     }
 
@@ -2962,7 +2984,9 @@ update_actions (WnckWindow *window)
       else
         {
           const char *name = _wnck_atom_name (atoms [i]);
-          g_warning ("Unhandled action type %s", name ? name: "(nil)");
+
+          if (name && g_str_has_prefix (name, "_NET_WM_"))
+            g_warning ("Unhandled action type %s", name);
         }
 
       i++;
@@ -3048,7 +3072,11 @@ update_wintype (WnckWindow *window)
       found_type = TRUE;
     }
 
-  window->priv->wintype = type;
+  if (window->priv->wintype != type)
+    {
+      window->priv->need_emit_type_changed = TRUE;
+      window->priv->wintype = type;
+    }
 }
 
 static void
@@ -3260,7 +3288,7 @@ force_update_now (WnckWindow *window)
   update_wmclass (window);
   update_wmhints (window);
   update_transient_for (window); /* wintype needs this to be first */
-  update_wintype (window);
+  update_wintype (window);   /* emits signals */
   update_wm_state (window);
   update_state (window);     /* must come after the above, since they affect
                               * our calculated state
@@ -3289,6 +3317,9 @@ force_update_now (WnckWindow *window)
 
   if (window->priv->need_emit_role_changed)
     emit_role_changed (window);
+
+  if (window->priv->need_emit_type_changed)
+    emit_type_changed (window);
 }
 
 
@@ -3392,3 +3423,12 @@ emit_role_changed (WnckWindow *window)
                  signals[ROLE_CHANGED],
                  0);
 }
+
+static void
+emit_type_changed (WnckWindow *window)
+{
+  window->priv->need_emit_type_changed = FALSE;
+  g_signal_emit (G_OBJECT (window),
+                 signals[TYPE_CHANGED],
+                 0);
+}
